@@ -1,13 +1,8 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Jordi Boggiano <j.boggiano@seld.be>
-// (c) Thomas Queiroz <thomasqueirozb@gmail.com>
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-/* last synced with: env (GNU coreutils) 8.13 */
-
 // spell-checker:ignore (ToDO) chdir execvp progname subcommand subcommands unsets setenv putenv spawnp SIGSEGV SIGBUS sigaction
 
 use clap::{crate_name, crate_version, Arg, ArgAction, Command};
@@ -23,6 +18,7 @@ use std::os::unix::process::ExitStatusExt;
 use std::process;
 use uucore::display::Quotable;
 use uucore::error::{UClapError, UResult, USimpleError, UUsageError};
+use uucore::line_ending::LineEnding;
 use uucore::{format_usage, help_about, help_section, help_usage, show_warning};
 
 const ABOUT: &str = help_about!("env.md");
@@ -31,7 +27,7 @@ const AFTER_HELP: &str = help_section!("after help", "env.md");
 
 struct Options<'a> {
     ignore_env: bool,
-    null: bool,
+    line_ending: LineEnding,
     running_directory: Option<&'a str>,
     files: Vec<&'a str>,
     unsets: Vec<&'a str>,
@@ -41,11 +37,11 @@ struct Options<'a> {
 
 // print name=value env pairs on screen
 // if null is true, separate pairs with a \0, \n otherwise
-fn print_env(null: bool) {
+fn print_env(line_ending: LineEnding) {
     let stdout_raw = io::stdout();
     let mut stdout = stdout_raw.lock();
     for (n, v) in env::vars() {
-        write!(stdout, "{}={}{}", n, v, if null { '\0' } else { '\n' }).unwrap();
+        write!(stdout, "{}={}{}", n, v, line_ending).unwrap();
     }
 }
 
@@ -64,7 +60,7 @@ fn parse_name_value_opt<'a>(opts: &mut Options<'a>, opt: &'a str) -> UResult<boo
 }
 
 fn parse_program_opt<'a>(opts: &mut Options<'a>, opt: &'a str) -> UResult<()> {
-    if opts.null {
+    if opts.line_ending == LineEnding::Nul {
         Err(UUsageError::new(
             125,
             "cannot specify --null (-0) with command".to_string(),
@@ -103,7 +99,7 @@ fn load_config_file(opts: &mut Options) -> UResult<()> {
 
 #[cfg(not(windows))]
 #[allow(clippy::ptr_arg)]
-fn build_command<'a, 'b>(args: &'a mut Vec<&'b str>) -> (Cow<'b, str>, &'a [&'b str]) {
+fn build_command<'a, 'b>(args: &'a Vec<&'b str>) -> (Cow<'b, str>, &'a [&'b str]) {
     let progname = Cow::from(args[0]);
     (progname, &args[1..])
 }
@@ -181,7 +177,7 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
     let matches = app.try_get_matches_from(args).with_exit_code(125)?;
 
     let ignore_env = matches.get_flag("ignore-environment");
-    let null = matches.get_flag("null");
+    let line_ending = LineEnding::from_zero_flag(matches.get_flag("null"));
     let running_directory = matches.get_one::<String>("chdir").map(|s| s.as_str());
     let files = match matches.get_many::<String>("file") {
         Some(v) => v.map(|s| s.as_str()).collect(),
@@ -194,7 +190,7 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
 
     let mut opts = Options {
         ignore_env,
-        null,
+        line_ending,
         running_directory,
         files,
         unsets,
@@ -302,10 +298,13 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
 
     if opts.program.is_empty() {
         // no program provided, so just dump all env vars to stdout
-        print_env(opts.null);
+        print_env(opts.line_ending);
     } else {
         // we need to execute a command
+        #[cfg(windows)]
         let (prog, args) = build_command(&mut opts.program);
+        #[cfg(not(windows))]
+        let (prog, args) = build_command(&opts.program);
 
         /*
          * On Unix-like systems Command::status either ends up calling either fork or posix_spawnp
