@@ -1,13 +1,8 @@
-// * This file is part of the uutils coreutils package.
-// *
-// * (c) 2014 T. Jameson Little <t.jameson.little@gmail.com>
-// * (c) 2020 nicoo <nicoo@debian.org>
-// *
-// * For the full copyright and license information, please view the LICENSE file
-// * that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
-use std::error::Error;
-use std::fmt::Write as FmtWrite;
 use std::io::BufRead;
 use std::io::{self, stdin, stdout, Write};
 
@@ -15,7 +10,7 @@ mod factor;
 use clap::{crate_version, Arg, ArgAction, Command};
 pub use factor::*;
 use uucore::display::Quotable;
-use uucore::error::UResult;
+use uucore::error::{set_exit_code, FromIo, UResult};
 use uucore::{format_usage, help_about, help_usage, show_error, show_warning};
 
 mod miller_rabin;
@@ -35,26 +30,27 @@ mod options {
 fn print_factors_str(
     num_str: &str,
     w: &mut io::BufWriter<impl io::Write>,
-    factors_buffer: &mut String,
     print_exponents: bool,
-) -> Result<(), Box<dyn Error>> {
-    num_str
-        .trim()
-        .parse::<u64>()
-        .map_err(|e| e.into())
-        .and_then(|x| {
-            factors_buffer.clear();
-            // If print_exponents is true, use the alternate format specifier {:#} from fmt to print the factors
-            // of x in the form of p^e.
-            if print_exponents {
-                writeln!(factors_buffer, "{}:{:#}", x, factor(x))?;
-            } else {
-                writeln!(factors_buffer, "{}:{}", x, factor(x))?;
-            }
-            w.write_all(factors_buffer.as_bytes())?;
-            w.flush()?;
-            Ok(())
-        })
+) -> io::Result<()> {
+    let x = match num_str.trim().parse::<u64>() {
+        Ok(x) => x,
+        Err(e) => {
+            // We return Ok() instead of Err(), because it's non-fatal and we should try the next
+            // number.
+            show_warning!("{}: {}", num_str.maybe_quote(), e);
+            set_exit_code(1);
+            return Ok(());
+        }
+    };
+
+    // If print_exponents is true, use the alternate format specifier {:#} from fmt to print the factors
+    // of x in the form of p^e.
+    if print_exponents {
+        writeln!(w, "{}:{:#}", x, factor(x))?;
+    } else {
+        writeln!(w, "{}:{}", x, factor(x))?;
+    }
+    w.flush()
 }
 
 #[uucore::main]
@@ -67,24 +63,27 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let stdout = stdout();
     // We use a smaller buffer here to pass a gnu test. 4KiB appears to be the default pipe size for bash.
     let mut w = io::BufWriter::with_capacity(4 * 1024, stdout.lock());
-    let mut factors_buffer = String::new();
 
     if let Some(values) = matches.get_many::<String>(options::NUMBER) {
         for number in values {
-            if let Err(e) = print_factors_str(number, &mut w, &mut factors_buffer, print_exponents)
-            {
-                show_warning!("{}: {}", number.maybe_quote(), e);
-            }
+            print_factors_str(number, &mut w, print_exponents)
+                .map_err_context(|| "write error".into())?;
         }
     } else {
         let stdin = stdin();
         let lines = stdin.lock().lines();
         for line in lines {
-            for number in line.unwrap().split_whitespace() {
-                if let Err(e) =
-                    print_factors_str(number, &mut w, &mut factors_buffer, print_exponents)
-                {
-                    show_warning!("{}: {}", number.maybe_quote(), e);
+            match line {
+                Ok(line) => {
+                    for number in line.split_whitespace() {
+                        print_factors_str(number, &mut w, print_exponents)
+                            .map_err_context(|| "write error".into())?;
+                    }
+                }
+                Err(e) => {
+                    set_exit_code(1);
+                    show_error!("error reading input: {}", e);
+                    return Ok(());
                 }
             }
         }
