@@ -1,18 +1,12 @@
-//  * This file is part of `fmt` from the uutils coreutils package.
-//  *
-//  * (c) kwantam <kwantam@gmail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) INFTY MULT accum breakwords linebreak linebreaking linebreaks linelen maxlength minlength nchars ostream overlen parasplit plass posn powf punct signum slen sstart tabwidth tlen underlen winfo wlen wordlen
 
-use std::cmp;
-use std::i64;
 use std::io::{BufWriter, Stdout, Write};
-use std::mem;
-
-use uucore::crash;
+use std::{cmp, i64, mem};
 
 use crate::parasplit::{ParaWords, Paragraph, WordInfo};
 use crate::FmtOptions;
@@ -27,7 +21,7 @@ struct BreakArgs<'a> {
 }
 
 impl<'a> BreakArgs<'a> {
-    fn compute_width<'b>(&self, winfo: &WordInfo<'b>, posn: usize, fresh: bool) -> usize {
+    fn compute_width(&self, winfo: &WordInfo, posn: usize, fresh: bool) -> usize {
         if fresh {
             0
         } else {
@@ -48,7 +42,7 @@ pub fn break_lines(
     ostream: &mut BufWriter<Stdout>,
 ) -> std::io::Result<()> {
     // indent
-    let p_indent = &para.indent_str[..];
+    let p_indent = &para.indent_str;
     let p_indent_len = para.indent_len;
 
     // words
@@ -57,14 +51,12 @@ pub fn break_lines(
 
     // the first word will *always* appear on the first line
     // make sure of this here
-    let (w, w_len) = match p_words_words.next() {
-        Some(winfo) => (winfo.word, winfo.word_nchars),
-        None => {
-            return ostream.write_all(b"\n");
-        }
+    let Some(winfo) = p_words_words.next() else {
+        return ostream.write_all(b"\n");
     };
+
     // print the init, if it exists, and get its length
-    let p_init_len = w_len
+    let p_init_len = winfo.word_nchars
         + if opts.crown || opts.tagged {
             // handle "init" portion
             ostream.write_all(para.init_str.as_bytes())?;
@@ -77,8 +69,9 @@ pub fn break_lines(
             // except that mail headers get no indent at all
             0
         };
+
     // write first word after writing init
-    ostream.write_all(w.as_bytes())?;
+    ostream.write_all(winfo.word.as_bytes())?;
 
     // does this paragraph require uniform spacing?
     let uniform = para.mail_header || opts.uniform;
@@ -105,15 +98,16 @@ fn break_simple<'a, T: Iterator<Item = &'a WordInfo<'a>>>(
     mut iter: T,
     args: &mut BreakArgs<'a>,
 ) -> std::io::Result<()> {
-    iter.try_fold((args.init_len, false), |l, winfo| {
-        accum_words_simple(args, l, winfo)
+    iter.try_fold((args.init_len, false), |(l, prev_punct), winfo| {
+        accum_words_simple(args, l, prev_punct, winfo)
     })?;
     args.ostream.write_all(b"\n")
 }
 
 fn accum_words_simple<'a>(
     args: &mut BreakArgs<'a>,
-    (l, prev_punct): (usize, bool),
+    l: usize,
+    prev_punct: bool,
     winfo: &'a WordInfo<'a>,
 ) -> std::io::Result<(usize, bool)> {
     // compute the length of this word, considering how tabs will expand at this position on the line
@@ -223,6 +217,7 @@ struct LineBreak<'a> {
     fresh: bool,
 }
 
+#[allow(clippy::cognitive_complexity)]
 fn find_kp_breakpoints<'a, T: Iterator<Item = &'a WordInfo<'a>>>(
     iter: T,
     args: &BreakArgs<'a>,
@@ -234,14 +229,14 @@ fn find_kp_breakpoints<'a, T: Iterator<Item = &'a WordInfo<'a>>>(
         linebreak: None,
         break_before: false,
         demerits: 0,
-        prev_rat: 0.0f32,
+        prev_rat: 0.0,
         length: args.init_len,
         fresh: false,
     }];
     // this vec holds the current active linebreaks; next_ holds the breaks that will be active for
     // the next word
-    let active_breaks = &mut vec![0];
-    let next_active_breaks = &mut vec![];
+    let mut active_breaks = vec![0];
+    let mut next_active_breaks = vec![];
 
     let stretch = (args.opts.width - args.opts.goal) as isize;
     let minlength = args.opts.goal - stretch as usize;
@@ -249,10 +244,7 @@ fn find_kp_breakpoints<'a, T: Iterator<Item = &'a WordInfo<'a>>>(
     let mut is_sentence_start = false;
     let mut least_demerits = 0;
     loop {
-        let w = match iter.next() {
-            None => break,
-            Some(w) => w,
-        };
+        let Some(w) = iter.next() else { break };
 
         // if this is the last word, we don't add additional demerits for this break
         let (is_last_word, is_sentence_end) = match iter.peek() {
@@ -274,6 +266,7 @@ fn find_kp_breakpoints<'a, T: Iterator<Item = &'a WordInfo<'a>>>(
         next_active_breaks.clear();
         // go through each active break, extending it and possibly adding a new active
         // break if we are above the minimum required length
+        #[allow(clippy::explicit_iter_loop)]
         for &i in active_breaks.iter() {
             let active = &mut linebreaks[i];
             // normalize demerits to avoid overflow, and record if this is the least
@@ -358,38 +351,36 @@ fn find_kp_breakpoints<'a, T: Iterator<Item = &'a WordInfo<'a>>>(
             least_demerits = cmp::max(ld_next, 0);
         }
         // swap in new list of active breaks
-        mem::swap(active_breaks, next_active_breaks);
+        mem::swap(&mut active_breaks, &mut next_active_breaks);
         // If this was the last word in a sentence, the next one must be the first in the next.
         is_sentence_start = is_sentence_end;
     }
 
     // return the best path
-    build_best_path(&linebreaks, active_breaks)
+    build_best_path(&linebreaks, &active_breaks)
 }
 
 fn build_best_path<'a>(paths: &[LineBreak<'a>], active: &[usize]) -> Vec<(&'a WordInfo<'a>, bool)> {
-    let mut breakwords = vec![];
     // of the active paths, we select the one with the fewest demerits
-    let mut best_idx = match active.iter().min_by_key(|&&a| paths[a].demerits) {
-        None => crash!(
-            1,
-            "Failed to find a k-p linebreak solution. This should never happen."
-        ),
-        Some(&s) => s,
-    };
-
-    // now, chase the pointers back through the break list, recording
-    // the words at which we should break
-    loop {
-        let next_best = &paths[best_idx];
-        match next_best.linebreak {
-            None => return breakwords,
-            Some(prev) => {
-                breakwords.push((prev, next_best.break_before));
-                best_idx = next_best.prev;
+    active
+        .iter()
+        .min_by_key(|&&a| paths[a].demerits)
+        .map(|&(mut best_idx)| {
+            let mut breakwords = vec![];
+            // now, chase the pointers back through the break list, recording
+            // the words at which we should break
+            loop {
+                let next_best = &paths[best_idx];
+                match next_best.linebreak {
+                    None => return breakwords,
+                    Some(prev) => {
+                        breakwords.push((prev, next_best.break_before));
+                        best_idx = next_best.prev;
+                    }
+                }
             }
-        }
-    }
+        })
+        .unwrap_or_default()
 }
 
 // "infinite" badness is more like (1+BAD_INFTY)^2 because of how demerits are computed
