@@ -1,20 +1,18 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Sunrin SHIMURA
-// Collaborator: Jian Zeng
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
 // spell-checker:ignore (paths) GPGHome findxs
 
-use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
+use clap::{builder::ValueParser, crate_version, Arg, ArgAction, ArgMatches, Command};
 use uucore::display::{println_verbatim, Quotable};
 use uucore::error::{FromIo, UError, UResult, UUsageError};
 use uucore::{format_usage, help_about, help_usage};
 
 use std::env;
 use std::error::Error;
+use std::ffi::OsStr;
 use std::fmt::Display;
 use std::io::ErrorKind;
 use std::iter;
@@ -38,6 +36,7 @@ static OPT_DRY_RUN: &str = "dry-run";
 static OPT_QUIET: &str = "quiet";
 static OPT_SUFFIX: &str = "suffix";
 static OPT_TMPDIR: &str = "tmpdir";
+static OPT_P: &str = "p";
 static OPT_T: &str = "t";
 
 static ARG_TEMPLATE: &str = "template";
@@ -117,97 +116,58 @@ impl Display for MkTempError {
 /// This provides a layer of indirection between the application logic
 /// and the argument parsing library `clap`, allowing each to vary
 /// independently.
-struct Options {
+#[derive(Clone)]
+pub struct Options {
     /// Whether to create a temporary directory instead of a file.
-    directory: bool,
+    pub directory: bool,
 
     /// Whether to just print the name of a file that would have been created.
-    dry_run: bool,
+    pub dry_run: bool,
 
     /// Whether to suppress file creation error messages.
-    quiet: bool,
+    pub quiet: bool,
 
     /// The directory in which to create the temporary file.
     ///
     /// If `None`, the file will be created in the current directory.
-    tmpdir: Option<String>,
+    pub tmpdir: Option<PathBuf>,
 
     /// The suffix to append to the temporary file, if any.
-    suffix: Option<String>,
+    pub suffix: Option<String>,
 
     /// Whether to treat the template argument as a single file path component.
-    treat_as_template: bool,
+    pub treat_as_template: bool,
 
     /// The template to use for the name of the temporary file.
-    template: String,
-}
-
-/// Decide whether the argument to `--tmpdir` should actually be the template.
-///
-/// This function is required to work around a limitation of `clap`,
-/// the command-line argument parsing library. In case the command
-/// line is
-///
-/// ```sh
-/// mktemp --tmpdir XXX
-/// ```
-///
-/// the program should behave like
-///
-/// ```sh
-/// mktemp --tmpdir=${TMPDIR:-/tmp} XXX
-/// ```
-///
-/// However, `clap` thinks that `XXX` is the value of the `--tmpdir`
-/// option. This function returns `true` in this case and `false`
-/// in all other cases.
-fn is_tmpdir_argument_actually_the_template(matches: &ArgMatches) -> bool {
-    if !matches.contains_id(ARG_TEMPLATE) {
-        if let Some(tmpdir) = matches.get_one::<String>(OPT_TMPDIR) {
-            if !Path::new(tmpdir).is_dir() && tmpdir.contains("XXX") {
-                return true;
-            }
-        }
-    }
-    false
+    pub template: String,
 }
 
 impl Options {
     fn from(matches: &ArgMatches) -> Self {
-        // Special case to work around a limitation of `clap`; see
-        // `is_tmpdir_argument_actually_the_template()` for more
-        // information.
-        //
-        // Fixed in clap 3
-        // See https://github.com/clap-rs/clap/pull/1587
-        let (tmpdir, template) = if is_tmpdir_argument_actually_the_template(matches) {
-            let tmpdir = Some(env::temp_dir().display().to_string());
-            let template = matches.get_one::<String>(OPT_TMPDIR).unwrap().to_string();
-            (tmpdir, template)
-        } else {
+        let tmpdir = matches
+            .get_one::<PathBuf>(OPT_TMPDIR)
+            .or_else(|| matches.get_one::<PathBuf>(OPT_P))
+            .cloned();
+        let (tmpdir, template) = match matches.get_one::<String>(ARG_TEMPLATE) {
             // If no template argument is given, `--tmpdir` is implied.
-            match matches.get_one::<String>(ARG_TEMPLATE) {
-                None => {
-                    let tmpdir = match matches.get_one::<String>(OPT_TMPDIR) {
-                        None => Some(env::temp_dir().display().to_string()),
-                        Some(tmpdir) => Some(tmpdir.to_string()),
-                    };
-                    let template = DEFAULT_TEMPLATE;
-                    (tmpdir, template.to_string())
-                }
-                Some(template) => {
-                    let tmpdir = if env::var(TMPDIR_ENV_VAR).is_ok() && matches.get_flag(OPT_T) {
-                        env::var(TMPDIR_ENV_VAR).ok()
-                    } else if matches.contains_id(OPT_TMPDIR) {
-                        matches.get_one::<String>(OPT_TMPDIR).map(String::from)
-                    } else if matches.get_flag(OPT_T) {
-                        // mktemp -t foo.xxx should export in TMPDIR
-                        Some(env::temp_dir().display().to_string())
-                    } else {
-                        matches.get_one::<String>(OPT_TMPDIR).map(String::from)
-                    };
-                    (tmpdir, template.to_string())
-                }
+            None => {
+                let tmpdir = Some(tmpdir.unwrap_or_else(env::temp_dir));
+                let template = DEFAULT_TEMPLATE;
+                (tmpdir, template.to_string())
+            }
+            Some(template) => {
+                let tmpdir = if env::var(TMPDIR_ENV_VAR).is_ok() && matches.get_flag(OPT_T) {
+                    env::var_os(TMPDIR_ENV_VAR).map(|t| t.into())
+                } else if tmpdir.is_some() {
+                    tmpdir
+                } else if matches.get_flag(OPT_T) || matches.contains_id(OPT_TMPDIR) {
+                    // If --tmpdir is given without an argument, or -t is given
+                    // export in TMPDIR
+                    Some(env::temp_dir())
+                } else {
+                    None
+                };
+                (tmpdir, template.to_string())
             }
         };
         Self {
@@ -234,7 +194,7 @@ impl Options {
 /// `num_rand_chars`.
 struct Params {
     /// The directory that will contain the temporary file.
-    directory: String,
+    directory: PathBuf,
 
     /// The (non-random) prefix of the temporary file.
     prefix: String,
@@ -339,7 +299,7 @@ impl Params {
         let num_rand_chars = j - i;
 
         Ok(Self {
-            directory,
+            directory: directory.into(),
             prefix,
             num_rand_chars,
             suffix,
@@ -349,8 +309,7 @@ impl Params {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let args = args.collect_lossy();
-
+    let args: Vec<_> = args.collect();
     let matches = match uu_app().try_get_matches_from(&args) {
         Ok(m) => m,
         Err(e) => {
@@ -372,9 +331,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     if env::var("POSIXLY_CORRECT").is_ok() {
         // If POSIXLY_CORRECT was set, template MUST be the last argument.
-        if is_tmpdir_argument_actually_the_template(&matches) || matches.contains_id(ARG_TEMPLATE) {
+        if matches.contains_id(ARG_TEMPLATE) {
             // Template argument was provided, check if was the last one.
-            if args.last().unwrap() != &options.template {
+            if args.last().unwrap() != OsStr::new(&options.template) {
                 return Err(Box::new(MkTempError::TooManyTemplates));
             }
         }
@@ -399,12 +358,13 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         exec(&tmpdir, &prefix, rand, &suffix, make_dir)
     };
 
-    if suppress_file_err {
+    let res = if suppress_file_err {
         // Mapping all UErrors to ExitCodes prevents the errors from being printed
         res.map_err(|e| e.code().into())
     } else {
         res
-    }
+    };
+    println_verbatim(res?).map_err_context(|| "failed to print directory name".to_owned())
 }
 
 pub fn uu_app() -> Command {
@@ -444,8 +404,16 @@ pub fn uu_app() -> Command {
                 .value_name("SUFFIX"),
         )
         .arg(
-            Arg::new(OPT_TMPDIR)
+            Arg::new(OPT_P)
                 .short('p')
+                .help("short form of --tmpdir")
+                .value_name("DIR")
+                .num_args(1)
+                .value_parser(ValueParser::path_buf())
+                .value_hint(clap::ValueHint::DirPath),
+        )
+        .arg(
+            Arg::new(OPT_TMPDIR)
                 .long(OPT_TMPDIR)
                 .help(
                     "interpret TEMPLATE relative to DIR; if DIR is not specified, use \
@@ -457,6 +425,10 @@ pub fn uu_app() -> Command {
                 // Allows use of default argument just by setting --tmpdir. Else,
                 // use provided input to generate tmpdir
                 .num_args(0..=1)
+                // Require an equals to avoid ambiguity if no tmpdir is supplied
+                .require_equals(true)
+                .overrides_with(OPT_P)
+                .value_parser(ValueParser::path_buf())
                 .value_hint(clap::ValueHint::DirPath),
         )
         .arg(
@@ -471,7 +443,7 @@ pub fn uu_app() -> Command {
         .arg(Arg::new(ARG_TEMPLATE).num_args(..=1))
 }
 
-pub fn dry_exec(tmpdir: &str, prefix: &str, rand: usize, suffix: &str) -> UResult<()> {
+fn dry_exec(tmpdir: &Path, prefix: &str, rand: usize, suffix: &str) -> UResult<PathBuf> {
     let len = prefix.len() + suffix.len() + rand;
     let mut buf = Vec::with_capacity(len);
     buf.extend(prefix.as_bytes());
@@ -481,7 +453,7 @@ pub fn dry_exec(tmpdir: &str, prefix: &str, rand: usize, suffix: &str) -> UResul
     // Randomize.
     let bytes = &mut buf[prefix.len()..prefix.len() + rand];
     rand::thread_rng().fill(bytes);
-    for byte in bytes.iter_mut() {
+    for byte in bytes {
         *byte = match *byte % 62 {
             v @ 0..=9 => v + b'0',
             v @ 10..=35 => v - 10 + b'a',
@@ -492,7 +464,7 @@ pub fn dry_exec(tmpdir: &str, prefix: &str, rand: usize, suffix: &str) -> UResul
     // We guarantee utf8.
     let buf = String::from_utf8(buf).unwrap();
     let tmpdir = Path::new(tmpdir).join(buf);
-    println_verbatim(tmpdir).map_err_context(|| "failed to print directory name".to_owned())
+    Ok(tmpdir)
 }
 
 /// Create a temporary directory with the given parameters.
@@ -506,7 +478,7 @@ pub fn dry_exec(tmpdir: &str, prefix: &str, rand: usize, suffix: &str) -> UResul
 ///
 /// If the temporary directory could not be written to disk or if the
 /// given directory `dir` does not exist.
-fn make_temp_dir(dir: &str, prefix: &str, rand: usize, suffix: &str) -> UResult<PathBuf> {
+fn make_temp_dir(dir: &Path, prefix: &str, rand: usize, suffix: &str) -> UResult<PathBuf> {
     let mut builder = Builder::new();
     builder.prefix(prefix).rand_bytes(rand).suffix(suffix);
     match builder.tempdir_in(dir) {
@@ -538,7 +510,7 @@ fn make_temp_dir(dir: &str, prefix: &str, rand: usize, suffix: &str) -> UResult<
 ///
 /// If the file could not be written to disk or if the directory does
 /// not exist.
-fn make_temp_file(dir: &str, prefix: &str, rand: usize, suffix: &str) -> UResult<PathBuf> {
+fn make_temp_file(dir: &Path, prefix: &str, rand: usize, suffix: &str) -> UResult<PathBuf> {
     let mut builder = Builder::new();
     builder.prefix(prefix).rand_bytes(rand).suffix(suffix);
     match builder.tempfile_in(dir) {
@@ -557,7 +529,7 @@ fn make_temp_file(dir: &str, prefix: &str, rand: usize, suffix: &str) -> UResult
     }
 }
 
-fn exec(dir: &str, prefix: &str, rand: usize, suffix: &str, make_dir: bool) -> UResult<()> {
+fn exec(dir: &Path, prefix: &str, rand: usize, suffix: &str, make_dir: bool) -> UResult<PathBuf> {
     let path = if make_dir {
         make_temp_dir(dir, prefix, rand, suffix)?
     } else {
@@ -576,7 +548,27 @@ fn exec(dir: &str, prefix: &str, rand: usize, suffix: &str, make_dir: bool) -> U
     // relative path.
     let path = Path::new(dir).join(filename);
 
-    println_verbatim(path).map_err_context(|| "failed to print directory name".to_owned())
+    Ok(path)
+}
+
+/// Create a temporary file or directory
+///
+/// Behavior is determined by the `options` parameter, see [`Options`] for details.
+pub fn mktemp(options: &Options) -> UResult<PathBuf> {
+    // Parse file path parameters from the command-line options.
+    let Params {
+        directory: tmpdir,
+        prefix,
+        num_rand_chars: rand,
+        suffix,
+    } = Params::from(options.clone())?;
+
+    // Create the temporary file or directory, or simulate creating it.
+    if options.dry_run {
+        dry_exec(&tmpdir, &prefix, rand, &suffix)
+    } else {
+        exec(&tmpdir, &prefix, rand, &suffix, options.directory)
+    }
 }
 
 #[cfg(test)]
