@@ -1,3 +1,7 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use std::str::from_utf8;
 
 use crate::formatteriteminfo::{FormatWriter, FormatterItemInfo};
@@ -82,7 +86,7 @@ pub fn format_ascii_dump(bytes: &[u8]) -> String {
     let mut result = String::new();
 
     result.push('>');
-    for c in bytes.iter() {
+    for c in bytes {
         if *c >= 0x20 && *c <= 0x7e {
             result.push_str(C_CHARS[*c as usize]);
         } else {
@@ -95,6 +99,7 @@ pub fn format_ascii_dump(bytes: &[u8]) -> String {
 }
 
 #[test]
+#[allow(clippy::cognitive_complexity)]
 fn test_format_item_a() {
     assert_eq!(" nul", format_item_a(0x00));
     assert_eq!(" soh", format_item_a(0x01));
@@ -110,6 +115,7 @@ fn test_format_item_a() {
 }
 
 #[test]
+#[allow(clippy::cognitive_complexity)]
 fn test_format_item_c() {
     assert_eq!("  \\0", format_item_c(&[0x00]));
     assert_eq!(" 001", format_item_c(&[0x01]));
