@@ -1,3 +1,7 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use std::ffi::OsString;
 use std::fmt::{Display, Formatter, Write};
 use std::io;
@@ -22,7 +26,7 @@ pub(crate) enum Error {
     #[error("No command is specified")]
     MissingCommand,
 
-    #[error("SELinux is not enabled")]
+    #[error("runcon may be used only on a SELinux kernel")]
     SELinuxNotEnabled,
 
     #[error(transparent)]
@@ -83,7 +87,6 @@ where
         err = source;
         write!(writer, ": {err}")?;
     }
-    write!(writer, ".")?;
     Ok(())
 }
 
