@@ -1,9 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Michael Debertol <michael.debertol..AT..gmail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 //! Sort big files by using auxiliary files for storing intermediate chunks.
 //!
@@ -84,11 +82,7 @@ fn reader_writer<
     output: Output,
     tmp_dir: &mut TmpDirWrapper,
 ) -> UResult<()> {
-    let separator = if settings.zero_terminated {
-        b'\0'
-    } else {
-        b'\n'
-    };
+    let separator = settings.line_ending.into();
 
     // Heuristically chosen: Dividing by 10 seems to keep our memory usage roughly
     // around settings.buffer_size as a whole.
@@ -230,7 +224,7 @@ fn read_write_loop<I: WriteableTmpFile>(
     let mut sender_option = Some(sender);
     let mut tmp_files = vec![];
     loop {
-        let mut chunk = match receiver.recv() {
+        let chunk = match receiver.recv() {
             Ok(it) => it,
             _ => {
                 return Ok(ReadResult::WroteChunksToFile { tmp_files });
@@ -238,7 +232,7 @@ fn read_write_loop<I: WriteableTmpFile>(
         };
 
         let tmp_file = write::<I>(
-            &mut chunk,
+            &chunk,
             tmp_dir.next_file()?,
             settings.compress_prog.as_deref(),
             separator,
@@ -268,7 +262,7 @@ fn read_write_loop<I: WriteableTmpFile>(
 /// Write the lines in `chunk` to `file`, separated by `separator`.
 /// `compress_prog` is used to optionally compress file contents.
 fn write<I: WriteableTmpFile>(
-    chunk: &mut Chunk,
+    chunk: &Chunk,
     file: (File, PathBuf),
     compress_prog: Option<&str>,
     separator: u8,
