@@ -1,9 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Michael Debertol <michael.debertol..AT..gmail.com>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 //! Fast comparison for strings representing a base 10 number without precision loss.
 //!
@@ -269,7 +267,7 @@ mod tests {
     fn parses_exp() {
         let n = "1";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 0,
@@ -280,7 +278,7 @@ mod tests {
         );
         let n = "100";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 2,
@@ -308,7 +306,7 @@ mod tests {
         );
         let n = "1,000";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 0,
@@ -319,7 +317,7 @@ mod tests {
         );
         let n = "1000.00";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 3,
@@ -333,7 +331,7 @@ mod tests {
     fn parses_negative_exp() {
         let n = "0.00005";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: -5,
@@ -344,7 +342,7 @@ mod tests {
         );
         let n = "00000.00005";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: -5,
@@ -359,7 +357,7 @@ mod tests {
     fn parses_sign() {
         let n = "5";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 0,
@@ -370,7 +368,7 @@ mod tests {
         );
         let n = "-5";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 0,
@@ -381,7 +379,7 @@ mod tests {
         );
         let n = "    -5";
         assert_eq!(
-            NumInfo::parse(n, &Default::default()),
+            NumInfo::parse(n, &NumInfoParseSettings::default()),
             (
                 NumInfo {
                     exponent: 0,
@@ -393,11 +391,11 @@ mod tests {
     }
 
     fn test_helper(a: &str, b: &str, expected: Ordering) {
-        let (a_info, a_range) = NumInfo::parse(a, &Default::default());
-        let (b_info, b_range) = NumInfo::parse(b, &Default::default());
+        let (a_info, a_range) = NumInfo::parse(a, &NumInfoParseSettings::default());
+        let (b_info, b_range) = NumInfo::parse(b, &NumInfoParseSettings::default());
         let ordering = numeric_str_cmp(
-            (&a[a_range.to_owned()], &a_info),
-            (&b[b_range.to_owned()], &b_info),
+            (&a[a_range.clone()], &a_info),
+            (&b[b_range.clone()], &b_info),
         );
         assert_eq!(ordering, expected);
         let ordering = numeric_str_cmp((&b[b_range], &b_info), (&a[a_range], &a_info));
@@ -469,7 +467,7 @@ mod tests {
     }
     #[test]
     fn single_minus() {
-        let info = NumInfo::parse("-", &Default::default());
+        let info = NumInfo::parse("-", &NumInfoParseSettings::default());
         assert_eq!(
             info,
             (
