@@ -1,7 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 //! Iterating over a file by chunks, either starting at the end of the file with [`ReverseChunks`]
 //! or at the end of piped stdin with [`LinesChunk`] or [`BytesChunk`].
@@ -495,7 +495,7 @@ impl LinesChunk {
     fn calculate_bytes_offset_from(&self, offset: usize) -> usize {
         let mut lines_offset = offset;
         let mut bytes_offset = 0;
-        for byte in self.get_buffer().iter() {
+        for byte in self.get_buffer() {
             if lines_offset == 0 {
                 break;
             }
