@@ -1,7 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) tailable untailable stdlib kqueue Uncategorized unwatch
 
@@ -10,7 +10,6 @@ use crate::follow::files::{FileHandling, PathData};
 use crate::paths::{Input, InputKind, MetadataExtTail, PathExtTail};
 use crate::{platform, text};
 use notify::{RecommendedWatcher, RecursiveMode, Watcher, WatcherKind};
-use std::collections::VecDeque;
 use std::io::BufRead;
 use std::path::{Path, PathBuf};
 use std::sync::mpsc::{self, channel, Receiver};
@@ -270,7 +269,7 @@ impl Observer {
         self.follow_name() && self.retry
     }
 
-    fn init_files(&mut self, inputs: &VecDeque<Input>) -> UResult<()> {
+    fn init_files(&mut self, inputs: &Vec<Input>) -> UResult<()> {
         if let Some(watcher_rx) = &mut self.watcher_rx {
             for input in inputs {
                 match input.kind() {
@@ -280,7 +279,7 @@ impl Observer {
                         if !path.is_file() {
                             continue;
                         }
-                        let mut path = path.to_owned();
+                        let mut path = path.clone();
                         if path.is_relative() {
                             path = std::env::current_dir()?.join(path);
                         }
@@ -303,6 +302,7 @@ impl Observer {
         Ok(())
     }
 
+    #[allow(clippy::cognitive_complexity)]
     fn handle_event(
         &mut self,
         event: &notify::Event,
@@ -345,7 +345,7 @@ impl Observer {
                                     show_error!("{}: file truncated", display_name);
                                     self.files.update_reader(event_path)?;
                                 }
-                                paths.push(event_path.to_owned());
+                                paths.push(event_path.clone());
                             } else if !is_tailable && old_md.is_tailable() {
                                 if pd.reader.is_some() {
                                     self.files.reset_reader(event_path);
@@ -359,7 +359,7 @@ impl Observer {
                         } else if is_tailable {
                                 show_error!( "{} has appeared;  following new file", display_name.quote());
                                 self.files.update_reader(event_path)?;
-                                paths.push(event_path.to_owned());
+                                paths.push(event_path.clone());
                             } else if settings.retry {
                                 if self.follow_descriptor() {
                                     show_error!(
@@ -403,7 +403,7 @@ impl Observer {
                                     "{} cannot be used, reverting to polling",
                                     text::BACKEND
                                 );
-                                self.orphans.push(event_path.to_owned());
+                                self.orphans.push(event_path.clone());
                                 let _ = self.watcher_rx.as_mut().unwrap().unwatch(event_path);
                             }
                         } else {
@@ -451,7 +451,7 @@ impl Observer {
 
                 if self.follow_descriptor() {
                     let new_path = event.paths.last().unwrap();
-                    paths.push(new_path.to_owned());
+                    paths.push(new_path.clone());
 
                     let new_data = PathData::from_other_with_path(self.files.remove(event_path), new_path);
                     self.files.insert(
@@ -471,6 +471,7 @@ impl Observer {
     }
 }
 
+#[allow(clippy::cognitive_complexity)]
 pub fn follow(mut observer: Observer, settings: &Settings) -> UResult<()> {
     if observer.files.no_files_remaining(settings) && !observer.files.only_stdin_remaining() {
         return Err(USimpleError::new(1, text::NO_FILES_REMAINING.to_string()));
@@ -478,8 +479,7 @@ pub fn follow(mut observer: Observer, settings: &Settings) -> UResult<()> {
 
     let mut process = platform::ProcessChecker::new(observer.pid);
 
-    let mut _event_counter = 0;
-    let mut _timeout_counter = 0;
+    let mut timeout_counter = 0;
 
     // main follow loop
     loop {
@@ -528,8 +528,7 @@ pub fn follow(mut observer: Observer, settings: &Settings) -> UResult<()> {
             .receiver
             .recv_timeout(settings.sleep_sec);
         if rx_result.is_ok() {
-            _event_counter += 1;
-            _timeout_counter = 0;
+            timeout_counter = 0;
         }
 
         let mut paths = vec![]; // Paths worth checking for new content to print
@@ -566,11 +565,11 @@ pub fn follow(mut observer: Observer, settings: &Settings) -> UResult<()> {
                     format!("{} resources exhausted", text::BACKEND),
                 ))
             }
-            Ok(Err(e)) => return Err(USimpleError::new(1, format!("NotifyError: {}", e))),
+            Ok(Err(e)) => return Err(USimpleError::new(1, format!("NotifyError: {e}"))),
             Err(mpsc::RecvTimeoutError::Timeout) => {
-                _timeout_counter += 1;
+                timeout_counter += 1;
             }
-            Err(e) => return Err(USimpleError::new(1, format!("RecvTimeoutError: {}", e))),
+            Err(e) => return Err(USimpleError::new(1, format!("RecvTimeoutError: {e}"))),
         }
 
         if observer.use_polling && settings.follow.is_some() {
@@ -585,7 +584,7 @@ pub fn follow(mut observer: Observer, settings: &Settings) -> UResult<()> {
             _read_some = observer.files.tail_file(path, settings.verbose)?;
         }
 
-        if _timeout_counter == settings.max_unchanged_stats {
+        if timeout_counter == settings.max_unchanged_stats {
             /*
             TODO: [2021-10; jhscheer] implement timeout_counter for each file.
             ‘--max-unchanged-stats=n’
