@@ -1,7 +1,5 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Daniel Rocco <drocco@gmail.com>
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
@@ -166,7 +164,7 @@ impl Parser {
     /// The stream is unchanged and will return the same Symbol on subsequent
     /// calls to `next()` or `peek()`.
     fn peek(&mut self) -> Symbol {
-        Symbol::new(self.tokens.peek().map(|s| s.to_os_string()))
+        Symbol::new(self.tokens.peek().cloned())
     }
 
     /// Test if the next token in the stream is a BOOLOP (-a or -o), without
