@@ -1,3 +1,7 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 //! Traits and implementations for iterating over lines in a file-like object.
 //!
 //! This module provides a [`WordCountable`] trait and implementations
@@ -13,12 +17,14 @@ use std::os::unix::io::AsRawFd;
 pub trait WordCountable: AsRawFd + Read {
     type Buffered: BufRead;
     fn buffered(self) -> Self::Buffered;
+    fn inner_file(&mut self) -> Option<&mut File>;
 }
 
 #[cfg(not(unix))]
 pub trait WordCountable: Read {
     type Buffered: BufRead;
     fn buffered(self) -> Self::Buffered;
+    fn inner_file(&mut self) -> Option<&mut File>;
 }
 
 impl WordCountable for StdinLock<'_> {
@@ -27,11 +33,19 @@ impl WordCountable for StdinLock<'_> {
     fn buffered(self) -> Self::Buffered {
         self
     }
+    fn inner_file(&mut self) -> Option<&mut File> {
+        None
+    }
 }
+
 impl WordCountable for File {
     type Buffered = BufReader<Self>;
 
     fn buffered(self) -> Self::Buffered {
         BufReader::new(self)
     }
+
+    fn inner_file(&mut self) -> Option<&mut File> {
+        Some(self)
+    }
 }
