@@ -1,7 +1,11 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore (words) nosuchgroup groupname
 
-use crate::common::util::*;
-use rust_users::*;
+use crate::common::util::TestScenario;
+use uucore::process::getegid;
 
 #[test]
 fn test_invalid_option() {
@@ -48,12 +52,12 @@ fn test_invalid_group() {
         .arg("__nosuchgroup__")
         .arg("/")
         .fails()
-        .stderr_is("chgrp: invalid group: '__nosuchgroup__'");
+        .stderr_is("chgrp: invalid group: '__nosuchgroup__'\n");
 }
 
 #[test]
 fn test_1() {
-    if get_effective_gid() != 0 {
+    if getegid() != 0 {
         new_ucmd!().arg("bin").arg(DIR).fails().stderr_contains(
             // linux fails with "Operation not permitted (os error 1)"
             // because of insufficient permissions,
@@ -66,7 +70,7 @@ fn test_1() {
 
 #[test]
 fn test_fail_silently() {
-    if get_effective_gid() != 0 {
+    if getegid() != 0 {
         for opt in ["-f", "--silent", "--quiet", "--sil", "--qui"] {
             new_ucmd!()
                 .arg(opt)
@@ -92,7 +96,7 @@ fn test_preserve_root() {
             .arg("-R")
             .arg("bin").arg(d)
             .fails()
-            .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe");
+            .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe\n");
     }
 }
 
@@ -111,16 +115,16 @@ fn test_preserve_root_symlink() {
             .arg("-HR")
             .arg("bin").arg(file)
             .fails()
-            .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe");
+            .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe\n");
     }
 
     let (at, mut ucmd) = at_and_ucmd!();
     at.symlink_file("///dev", file);
     ucmd.arg("--preserve-root")
         .arg("-HR")
-        .arg("bin").arg(format!(".//{}/..//..//../../", file))
+        .arg("bin").arg(format!(".//{file}/..//..//../../"))
         .fails()
-        .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe");
+        .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe\n");
 
     let (at, mut ucmd) = at_and_ucmd!();
     at.symlink_file("/", "__root__");
@@ -128,7 +132,7 @@ fn test_preserve_root_symlink() {
         .arg("-R")
         .arg("bin").arg("__root__/.")
         .fails()
-        .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe");
+        .stderr_is("chgrp: it is dangerous to operate recursively on '/'\nchgrp: use --no-preserve-root to override this failsafe\n");
 }
 
 #[test]
@@ -137,13 +141,13 @@ fn test_reference() {
     // skip for root or MS-WSL
     // * MS-WSL is bugged (as of 2019-12-25), allowing non-root accounts su-level privileges for `chgrp`
     // * for MS-WSL, succeeds and stdout == 'group of /etc retained as root'
-    if !(get_effective_gid() == 0 || uucore::os::is_wsl_1()) {
+    if !(getegid() == 0 || uucore::os::is_wsl_1()) {
         new_ucmd!()
             .arg("-v")
             .arg("--reference=/etc/passwd")
             .arg("/etc")
             .fails()
-            .stderr_is("chgrp: changing group of '/etc': Operation not permitted (os error 1)\nfailed to change group of '/etc' from root to root");
+            .stderr_is("chgrp: changing group of '/etc': Operation not permitted (os error 1)\nfailed to change group of '/etc' from root to root\n");
     }
 }
 
@@ -196,14 +200,14 @@ fn test_missing_files() {
         .arg("groupname")
         .fails()
         .stderr_contains(
-            "error: The following required arguments were not provided:\n  <FILE>...\n",
+            "error: the following required arguments were not provided:\n  <FILE>...\n",
         );
 }
 
 #[test]
 #[cfg(target_os = "linux")]
 fn test_big_p() {
-    if get_effective_gid() != 0 {
+    if getegid() != 0 {
         new_ucmd!()
             .arg("-RP")
             .arg("bin")
@@ -218,7 +222,7 @@ fn test_big_p() {
 #[test]
 #[cfg(any(target_os = "linux", target_os = "android"))]
 fn test_big_h() {
-    if get_effective_gid() != 0 {
+    if getegid() != 0 {
         assert!(
             new_ucmd!()
                 .arg("-RH")
@@ -270,7 +274,7 @@ fn test_permission_denied() {
             .arg(group.as_raw().to_string())
             .arg("dir")
             .fails()
-            .stderr_only("chgrp: cannot access 'dir': Permission denied");
+            .stderr_only("chgrp: cannot access 'dir': Permission denied\n");
     }
 }
 
@@ -289,7 +293,7 @@ fn test_subdir_permission_denied() {
             .arg(group.as_raw().to_string())
             .arg("dir")
             .fails()
-            .stderr_only("chgrp: cannot access 'dir/subdir': Permission denied");
+            .stderr_only("chgrp: cannot access 'dir/subdir': Permission denied\n");
     }
 }
 
