@@ -1,7 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 use crate::common::util::{expected_result, TestScenario};
 use pinky::Capitalize;
@@ -21,6 +21,7 @@ fn test_capitalize() {
 }
 
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_long_format() {
     let login = "root";
     let pw: Passwd = Passwd::locate(login).unwrap();
@@ -44,6 +45,7 @@ fn test_long_format() {
 
 #[cfg(unix)]
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_long_format_multiple_users() {
     // multiple instances of one account we know exists,
     // the account of the test runner,
@@ -71,6 +73,7 @@ fn test_long_format_wo_user() {
 
 #[cfg(unix)]
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_short_format_i() {
     // allow whitespace variation
     // * minor whitespace differences occur between platform built-in outputs; specifically, the number of trailing TABs may be variant
@@ -85,6 +88,7 @@ fn test_short_format_i() {
 
 #[cfg(unix)]
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_short_format_q() {
     // allow whitespace variation
     // * minor whitespace differences occur between platform built-in outputs; specifically, the number of trailing TABs may be variant
@@ -99,6 +103,7 @@ fn test_short_format_q() {
 
 #[cfg(unix)]
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_no_flag() {
     let ts = TestScenario::new(util_name!());
     let actual = ts.ucmd().succeeds().stdout_move_str();
