@@ -1,3 +1,7 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use crate::common::util::TestScenario;
 
 #[test]
@@ -71,7 +75,7 @@ fn test_echo() {
 #[test]
 fn test_head_count() {
     let repeat_limit = 5;
-    let input_seq = vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
+    let input_seq = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
     let input = input_seq
         .iter()
         .map(ToString::to_string)
@@ -102,7 +106,7 @@ fn test_head_count() {
 #[test]
 fn test_repeat() {
     let repeat_limit = 15000;
-    let input_seq = vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
+    let input_seq = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
     let input = input_seq
         .iter()
         .map(ToString::to_string)
