@@ -1,4 +1,10 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use crate::common::util::TestScenario;
+#[cfg(target_os = "linux")]
+use std::fmt::Write;
 
 // tests for basic tee functionality.
 // inspired by:
@@ -70,7 +76,10 @@ fn test_tee_append() {
 fn test_tee_no_more_writeable_1() {
     // equals to 'tee /dev/full out2 <multi_read' call
     let (at, mut ucmd) = at_and_ucmd!();
-    let content = (1..=10).map(|x| format!("{x}\n")).collect::<String>();
+    let content = (1..=10).fold(String::new(), |mut output, x| {
+        let _ = writeln!(output, "{x}");
+        output
+    });
     let file_out = "tee_file_out";
 
     ucmd.arg("/dev/full")
@@ -90,7 +99,10 @@ fn test_tee_no_more_writeable_2() {
     // but currently there is no way to redirect stdout to /dev/full
     // so this test is disabled
     let (_at, mut ucmd) = at_and_ucmd!();
-    let _content = (1..=10).map(|x| format!("{x}\n")).collect::<String>();
+    let _content = (1..=10).fold(String::new(), |mut output, x| {
+        let _ = writeln!(output, "{x}");
+        output
+    });
     let file_out_a = "tee_file_out_a";
     let file_out_b = "tee_file_out_b";
 
@@ -110,6 +122,7 @@ fn test_tee_no_more_writeable_2() {
 mod linux_only {
     use crate::common::util::{AtPath, TestScenario, UCommand};
 
+    use std::fmt::Write;
     use std::fs::File;
     use std::process::{Output, Stdio};
 
@@ -131,7 +144,10 @@ mod linux_only {
     }
 
     fn run_tee(proc: &mut UCommand) -> (String, Output) {
-        let content = (1..=100_000).map(|x| format!("{x}\n")).collect::<String>();
+        let content = (1..=100_000).fold(String::new(), |mut output, x| {
+            let _ = writeln!(output, "{x}");
+            output
+        });
 
         #[allow(deprecated)]
         let output = proc
