@@ -1,5 +1,9 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 // spell-checker:ignore aabbaa aabbcc aabc abbb abcc abcdefabcdef abcdefghijk abcdefghijklmn abcdefghijklmnop ABCDEFGHIJKLMNOPQRS abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ ABCDEFZZ abcxyz ABCXYZ abcxyzabcxyz ABCXYZABCXYZ acbdef alnum amzamz AMZXAMZ bbbd cclass cefgm cntrl compl dabcdef dncase Gzabcdefg PQRST upcase wxyzz xdigit xycde xyyye xyyz xyzzzzxyzzzz ZABCDEF Zamz Cdefghijkl Cdefghijklmn
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 #[test]
 fn test_invalid_arg() {
@@ -25,12 +29,12 @@ fn test_small_set2() {
 }
 
 #[test]
-fn test_unicode() {
+fn test_invalid_unicode() {
     new_ucmd!()
-        .args(&[", ┬─┬", "╯︵┻━┻"])
-        .pipe_in("(,°□°）, ┬─┬")
-        .run()
-        .stdout_is("(╯°□°）╯︵┻━┻");
+        .args(&["-dc", "abc"])
+        .pipe_in([0o200, b'a', b'b', b'c'])
+        .succeeds()
+        .stdout_is("abc");
 }
 
 #[test]
@@ -729,10 +733,11 @@ fn check_against_gnu_tr_tests_w() {
     //  {IN=>"\300\301\377\345\345\350\345"},
     //  {OUT=>"\300\301\377\345"}],
     new_ucmd!()
-        .args(&["-ds", "\u{350}", "\u{345}"])
-        .pipe_in("\u{300}\u{301}\u{377}\u{345}\u{345}\u{350}\u{345}")
+        .arg("-ds")
+        .args(&["\\350", "\\345"])
+        .pipe_in([0o300, 0o301, 0o377, 0o345, 0o345, 0o350, 0o345])
         .succeeds()
-        .stdout_is("\u{300}\u{301}\u{377}\u{345}");
+        .stdout_is_bytes([0o300, 0o301, 0o377, 0o345]);
 }
 
 #[test]
@@ -925,7 +930,7 @@ fn check_against_gnu_tr_tests_bs_at_end() {
         .pipe_in(r"\")
         .succeeds()
         .stdout_is("x")
-        .stderr_is("tr: warning: an unescaped backslash at end of string is not portable");
+        .stderr_is("tr: warning: an unescaped backslash at end of string is not portable\n");
 }
 
 #[test]
@@ -938,7 +943,7 @@ fn check_against_gnu_tr_tests_ross_0a() {
         .args(&["-cs", "[:upper:]", "X[Y*]"])
         .pipe_in("")
         .fails()
-        .stderr_is("tr: when translating with complemented character classes,\nstring2 must map all characters in the domain to one");
+        .stderr_is("tr: when translating with complemented character classes,\nstring2 must map all characters in the domain to one\n");
 }
 
 #[test]
@@ -1050,6 +1055,24 @@ fn check_against_gnu_tr_tests_empty_cc() {
         .stderr_is("tr: missing character class name '[::]'\n");
 }
 
+#[test]
+fn check_against_gnu_tr_tests_repeat_set1() {
+    new_ucmd!()
+        .args(&["[a*]", "a"])
+        .pipe_in("")
+        .fails()
+        .stderr_is("tr: the [c*] repeat construct may not appear in string1\n");
+}
+
+#[test]
+fn check_against_gnu_tr_tests_repeat_set2() {
+    new_ucmd!()
+        .args(&["a", "[a*][a*]"])
+        .pipe_in("")
+        .fails()
+        .stderr_is("tr: only one [c*] repeat construct may appear in string2\n");
+}
+
 #[test]
 fn check_against_gnu_tr_tests_repeat_bs_9() {
     // # Weird repeat counts.
