@@ -1,3 +1,8 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
+// spell-checker:ignore contenta
 use crate::common::util::TestScenario;
 
 #[test]
@@ -231,3 +236,34 @@ fn test_tabs_shortcut_with_too_large_size() {
 
     new_ucmd!().arg(arg).fails().stderr_contains(expected_error);
 }
+
+#[test]
+fn test_is_directory() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir_name = "dir";
+    at.mkdir(dir_name);
+
+    ucmd.arg(dir_name)
+        .fails()
+        .stderr_contains(format!("unexpand: {}: Is a directory", dir_name));
+}
+
+#[test]
+fn test_multiple_files() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.write("file", "content");
+    at.write("file1", "a        b");
+
+    ucmd.args(&["file", "file1"])
+        .succeeds()
+        .stdout_is("contenta        b");
+}
+
+#[test]
+fn test_one_nonexisting_file() {
+    new_ucmd!()
+        .arg("asdf.txt")
+        .fails()
+        .stderr_contains("asdf.txt: No such file or directory");
+}
