@@ -1,7 +1,11 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use std::io::Write;
 
 // spell-checker:ignore nabcd
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 static INPUT: &str = "sorted.txt";
 static OUTPUT: &str = "sorted-output.txt";
@@ -76,7 +80,7 @@ fn test_stdin_skip_and_check_2_chars() {
 }
 
 #[test]
-fn test_stdin_skip_1_field() {
+fn test_stdin_skip_2_fields() {
     new_ucmd!()
         .args(&["-f2"])
         .pipe_in_fixture(SKIP_FIELDS)
@@ -84,6 +88,42 @@ fn test_stdin_skip_1_field() {
         .stdout_is_fixture("skip-2-fields.expected");
 }
 
+#[test]
+fn test_stdin_skip_2_fields_obsolete() {
+    new_ucmd!()
+        .args(&["-2"])
+        .pipe_in_fixture(SKIP_FIELDS)
+        .run()
+        .stdout_is_fixture("skip-2-fields.expected");
+}
+
+#[test]
+fn test_stdin_skip_21_fields() {
+    new_ucmd!()
+        .args(&["-f21"])
+        .pipe_in_fixture(SKIP_FIELDS)
+        .run()
+        .stdout_is_fixture("skip-21-fields.expected");
+}
+
+#[test]
+fn test_stdin_skip_21_fields_obsolete() {
+    new_ucmd!()
+        .args(&["-21"])
+        .pipe_in_fixture(SKIP_FIELDS)
+        .run()
+        .stdout_is_fixture("skip-21-fields.expected");
+}
+
+#[test]
+fn test_stdin_skip_invalid_fields_obsolete() {
+    new_ucmd!()
+        .args(&["-5deadbeef"])
+        .run()
+        .failure()
+        .stderr_only("uniq: Invalid argument for skip-fields: 5deadbeef\n");
+}
+
 #[test]
 fn test_stdin_all_repeated() {
     new_ucmd!()
@@ -168,8 +208,8 @@ fn test_invalid_utf8() {
         .run()
         .failure()
         .stderr_only(
-            "uniq: failed to convert line to utf8: invalid utf-8 sequence of 1 bytes from index 0",
-        );
+        "uniq: failed to convert line to utf8: invalid utf-8 sequence of 1 bytes from index 0\n",
+    );
 }
 
 #[test]
@@ -432,15 +472,15 @@ fn gnu_tests() {
             stderr: None,
             exit: None,
         },
-        // // Obsolete syntax for "-f 1"
-        // TestCase {
-        //     name: "obs30",
-        //     args: &["-1"],
-        //     input: "a a\nb a\n",
-        //     stdout: Some("a a\n"),
-        //     stderr: None,
-        //     exit: None,
-        // },
+        // Obsolete syntax for "-f 1"
+        TestCase {
+            name: "obs30",
+            args: &["-1"],
+            input: "a a\nb a\n",
+            stdout: Some("a a\n"),
+            stderr: None,
+            exit: None,
+        },
         TestCase {
             name: "31",
             args: &["-f", "1"],
@@ -514,23 +554,25 @@ fn gnu_tests() {
             stderr: None,
             exit: None,
         },
-        // // Obsolete syntax for "-s 1"
-        // TestCase {
-        //     name: "obs-plus44",
-        //     args: &["+1", "--"],
-        //     input: "aaa\naaa\n",
-        //     stdout: Some("aaa\n"),
-        //     stderr: None,
-        //     exit: None,
-        // },
-        // TestCase {
-        //     name: "obs-plus45",
-        //     args: &["+1", "--"],
-        //     input: "baa\naaa\n",
-        //     stdout: Some("baa\n"),
-        //     stderr: None,
-        //     exit: None,
-        // },
+        /*
+        // Obsolete syntax for "-s 1"
+        TestCase {
+            name: "obs-plus44",
+            args: &["+1", "--"],
+            input: "aaa\naaa\n",
+            stdout: Some("aaa\n"),
+            stderr: None,
+            exit: None,
+        },
+        TestCase {
+            name: "obs-plus45",
+            args: &["+1", "--"],
+            input: "baa\naaa\n",
+            stdout: Some("baa\n"),
+            stderr: None,
+            exit: None,
+        },
+        */
         TestCase {
             name: "50",
             args: &["-f", "1", "-s", "1"],
